@@ -12,12 +12,17 @@ if len(args) != 3:
 def readRawFile():
 	rawdata = []
 	for line in open(args[1], "rt").readlines():
-		lookfor = ["basic_cpuid", "ext_cpuid", "intel_fn4", "intel_fn11", "amd_fn8000001dh"]
+		lookfor = ["basic_cpuid", "ext_cpuid", "intel_fn4", "intel_fn11", "amd_fn8000001dh", "Logical CPU", "CPUID"]
+		ignore  = ["MSR Register"]
 		good = False
 		for match in lookfor:
 			if line.find(match) != -1:
 				good = True
 				break
+		for match in ignore:
+			if line.find(match) != -1:
+				good = False
+				break
 		if good:
 			rawdata.append(line.strip())
 	return rawdata
@@ -30,16 +35,26 @@ def readResultFile():
 		if s.find(":") == -1:
 			continue
 		numeric = ["family", "model", "stepping", "ext_family", "ext_model",
-				"num_cores", "num_logical", "L1 D cache", "L1 I cache",
-				"L2 cache", "L3 cache", "L4 cache", "L1D assoc.", "L1I assoc.",
-				"L2 assoc.", "L3 assoc.", "L4 assoc.", "L1D line sz", "L1I line sz",
-				"L2 line sz", "L3 line sz", "L4 line sz"]
+				"num_cores", "num_logical",
+				"L1 D cache",  "L1 I cache",  "L2 cache",   "L3 cache",   "L4 cache",
+				"L1D assoc.",  "L1I assoc.",  "L2 assoc.",  "L3 assoc.",  "L4 assoc.",
+				"L1D line sz", "L1I line sz", "L2 line sz", "L3 line sz", "L4 line sz",
+				"L1D inst.",   "L1I inst.",   "L2 inst.",   "L3 inst.",   "L4 inst.",
+				]
 		field = s[:s.find(":")].strip()
 		if field in numeric:
 			value = s[s.find(":")+1:].strip()
 			if not rexp.match(value):
 				raise "Bad format of value: [%s]" % s
 			repdata.append(rexp.findall(value)[0])
+		if "CPU Info for type" in field:
+			repdata.append(delimiter)
+		if field == "arch":
+			value = s[s.find(":") + 2:]
+			repdata.append(value)
+		if field == "purpose":
+			value = s[s.find(":") + 2:]
+			repdata.append(value)
 		if field == "code name":
 			value = s[s.find("`") + 1: s.find("'")]
 			repdata.append(value)
@@ -56,5 +71,5 @@ def readResultFile():
 	return repdata
 
 delimiter = "-" * 80
-lines = readRawFile() + [delimiter] + readResultFile()
+lines = readRawFile() + readResultFile()
 sys.stdout.writelines([s + "\n" for s in lines])
